/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Gerben Venekamp <venekamp@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */


/*!
    \file   lcmaps.h
    \brief  Basic interface of the LCMAPS library.
    \author Martijn Steenbakkers for the EU DataGrid.
    \author Oscar Koeroo for the EGEE project.

    In order to use this API, please set in your application the following
    defines:
        LCMAPS_GSI_MODE:         If set, the GSI dependent interface can be used
        ALLOW_EMPTY_CREDENTIALS: If set, the api allows a DN to be used instead of a
                                 gss_credential (it is recommended to define this macro)

    This header contains the declarations of the LCMAPS library functions:
    -# lcmaps_init():
        To initialize the LCMAPS module
    -# lcmaps_init_and_log():
        To initialize the LCMAPS module and select logging type
    -# lcmaps_init_and_logfile():
        To initialize the LCMAPS module, select logging type and set logfile
    -# lcmaps_run_without_credentials_and_return_username():
        To do the user mapping, without credentials and return the user name
    -# lcmaps_run_with_fqans_mapcounter_and_return_account()
        To do the user mapping, based on DN+FQANs+mapcounter and return the account information
    -# lcmaps_run_with_fqans_and_return_account()
        To do the user mapping, based on DN+FQANs and return the account information
    -# lcmaps_run():
        To do the user mapping
    -# lcmaps_run_and_return_username():
        To do the user mapping and return the user name
    -# lcmaps_run_and_return_poolindex():
        To do the user mapping and return the poolindex
    -# lcmaps_term():
        To cleanly terminate the module
    -# lcmaps_run_with_stack_of_x509_and_return_account():
        To do the user mapping, based on a certificate chain and return the account information
    \ingroup LcmapsInterface
*/
/*!
    \defgroup LcmapsInterface The LCMAPS API for LCMAPS clients
    \brief The API is available by including the header lcmaps.h
*/

#ifndef LCMAPS_H
#define LCMAPS_H

#ifndef LCMAPS_C
#   define LCMAPS_C_STORAGE_CLASS extern
#else
#   define LCMAPS_C_STORAGE_CLASS
#endif


/******************************************************************************
                             Include header files
******************************************************************************/
#ifdef LCMAPS_GSI_MODE
#   include <openssl/x509.h>
#   include <gssapi.h>
#endif
#include "lcmaps_types.h"

/******************************************************************************
 *                 Module definition
 *****************************************************************************/


/******************************************************************************
Function:   lcmaps_init
Description:
    Initialize LCMAPS module: 
    setup logging, error handling
    start PluginManager

Parameters:
    fp: file handle for logging (from gatekeeper)
Returns:
    0: initialization succeeded
    1: initialization failed
******************************************************************************/
/*!
    \fn lcmaps_init(
        FILE* fp
        )
    \brief Initialize the LCMAPS module.

    The function does the following:
    - initialize LCMAPS module.
    - setup logging, error handling (not yet).
    - start PluginManager

    \param fp file handle for logging (from gatekeeper or other previously opened file handle)
              If the file handle is zero, assume that only syslogging is requested
    \retval 0 initialization succeeded.
    \retval 1 initialization failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_init(
        FILE* fp
);

/******************************************************************************
Function:   lcmaps_init_and_log
Description:
    Select logging type
    Start PluginManager:
    read from LCMAPS config file, the plugins to be loaded

Parameters:
    fp: file handle for logging (from gatekeeper or other previously opened file handle)
        If the file handle is zero, assume that only syslogging is requested
    logtype: type of logging (usrlog and/or syslog)

Returns:
    0: initialization succeeded
    1: initialization failed
******************************************************************************/
/*!
    \fn lcmaps_init_and_log(
        FILE* fp,
        unsigned short logtype
        )
    \brief Initialize the LCMAPS module and select logging type

    The function does the following:
    - initialize LCMAPS module.
    - setup logging, error handling (not yet).
    - start PluginManager

    \param fp file handle for logging (from gatekeeper or other previously opened file handle)
              If the file handle is zero, assume that only syslogging is requested
    \param logtype type of logging (usrlog and/or syslog)
    \retval 0 initialization succeeded.
    \retval 1 initialization failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_init_and_log(
        FILE* fp,
        unsigned short logtype
);

/******************************************************************************
Function:   lcmaps_init_and_logfile
Description:
    Select logging type
    Setup logging by providing a file handle or file name, error handling (not yet).
    Start PluginManager:
    read from LCMAPS config file, the plugins to be loaded

Parameters:
    logfile: name of logfile 
    fp: file handle for logging (from gatekeeper or other previously opened file handle)
        If the file handle is zero, assume that only syslogging is requested
    logtype: type of logging (usrlog and/or syslog)

Returns:
    0: initialization succeeded
    1: initialization failed
******************************************************************************/
/*!
    \fn lcmaps_init_and_logfile(
        char * logfile,
        FILE* fp,
        unsigned short logtype
        )
    \brief Initialize the LCMAPS module, select logging type and set logfile

    The function does the following:
    - initialize LCMAPS module.
    - Setup logging by providing a file handle or file name, error handling (not yet).
    - start PluginManager

    \param logfile name of logfile 
    \param fp file handle for logging (from gatekeeper or other previously opened file handle)
              If the file handle is zero, assume that only syslogging is requested
    \param logtype type of logging (usrlog and/or syslog)
    \retval 0 initialization succeeded.
    \retval 1 initialization failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_init_and_logfile(
        char * logfile,
        FILE* fp,
        unsigned short logtype
);


/******************************************************************************
Function:   lcmaps_term
Description:
    Terminate LCMAPS module: 

Parameters:
Returns:
    0: termination succeeded
    1: termination failed
******************************************************************************/
/*!
    \fn lcmaps_term()
    \brief Terminate the LCMAPS module.

    The function does the following:
    - terminate the LCMAPS module
    - terminate the plugins

    \retval 0 termination succeeded.
    \retval 1 termination failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_term(void);


/******************************************************************************
Function:   lcmaps_run_without_credentials_and_return_username
Description:
    Based on the only the user DN do the following:
    Do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return user name.
    This interface can be used to provide the legacy
        GLOBUS_GSS_ASSIST_GRIDMAP()
    interface.

Parameters:
    user_dn_tmp : user DN
    request     : RSL string
    usernamep   : pointer to user name (to be freed by calling application)
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation

Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
/*!
    \fn lcmaps_run_without_credentials_and_return_username(
        char * user_dn_tmp,
        lcmaps_request_t request,
        char ** usernamep,
        int npols,
        char ** policynames
    )
    \brief do the user mapping using only the user DN and return the username

    Based on the only the user DN do the following:
    Do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return user name.
    This interface can be used to provide the legacy
        \b globus_gss_assist_gridmap()
    interface.

    \param user_dn_tmp user DN
    \param request     authorization request as an RSL string
    \param usernamep   pointer to user name (to be freed by calling application).
                       Note: usernamep should be non-NULL at the start !
    \param npols       number of policies to be considered for evaluation
    \param policynames the names of the policies to be considered for evaluation

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_without_credentials_and_return_username(
        char * user_dn_tmp,
        lcmaps_request_t request,
        char ** usernamep,
        int npols,
        char ** policynames
);

/******************************************************************************
Function:   lcmaps_run_with_fqans_mapcounter_and_return_account
Description:
    LCMAPS will run with a list of FQANs and the DN as an input. In addition a
    list of policies may be provided.
    The allocated uid, gids and the poolindex will be returned to the calling
    application.
    This interface is intended to be used by a wrapper function for the 
        DYNAMIC ACCOUNTS SERVICE (DAS).

Parameters:
    user_dn     : the DN of the user
    fqan_list   : the list of (VOMS) FQANs that have been asserted to the user
    nfqan       : the number of FQANs in fqan_list
    mapcounter: : The counter which will be added to the poolindex, effectively enabling
                  multiple account mappings
    request     : RSL string
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation
    puid        : pointer to the uid found (output parameter)
    ppgid_list  : pointer to the list of primary gids found (output parameter)
    pnpgid      : pointer to the number of primary gids found (output parameter)
    psgid_list  : pointer to the list of secondary gids found (output parameter)
    pnsgid      : pointer to the number of secondary gids found (output parameter)
    poolindexp  : pointer to poolindex string (output parameter)

Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
/*!
    \fn lcmaps_run_with_fqans_mapcounter_and_return_account(
        char *            user_dn,
        char **           fqan_list,
        int               nfqan,
        int               mapcounter,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames,
        uid_t *           puid,
        gid_t **          ppgid_list,
        int *             pnpgid,
        gid_t **          psgid_list,
        int *             pnsgid,
        char **           poolindexp
    )
    \brief let LCMAPS handle the user mapping based on fqans and
           return the account information

    LCMAPS will run with a list of FQANs and the DN as an input. In addition a
    list of policies may be provided.
    The allocated uid, gids and the poolindex will be returned to the calling
    application.
    This interface is intended to be used by a wrapper function for the 
        \b WorkSpace \b Service (WSS, f.q.a. Dynamic Account Service - DAS).

    \param user_dn     the DN of the user
    \param fqan_list   the list of (VOMS) FQANs that have been asserted to the user
    \param nfqan       the number of FQANs in fqan_list
    \param mapcounter  the counter which will be added to the poolindex, effectively enabling
                       multiple account mappings
    \param request     RSL string
    \param npols       number of policies to be considered for evaluation
    \param policynames the names of the policies to be considered for evaluation
    \param puid        pointer to the uid found (output parameter)
    \param ppgid_list  pointer to the list of primary gids found (output parameter)
    \param pnpgid      pointer to the number of primary gids found (output parameter)
    \param psgid_list  pointer to the list of secondary gids found (output parameter)
    \param pnsgid      pointer to the number of secondary gids found (output parameter)
    \param poolindexp  pointer to poolindex string (output parameter)

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_with_fqans_mapcounter_and_return_account(
        char *            user_dn,
        char **           fqan_list,
        int               nfqan,
        int               mapcounter,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames,
        uid_t *           puid,
        gid_t **          ppgid_list,
        int *             pnpgid,
        gid_t **          psgid_list,
        int *             pnsgid,
        char **           poolindexp
);

/******************************************************************************
Function:   lcmaps_run_with_fqans_and_return_account
Description:
    LCMAPS will run with a list of FQANs and the DN as an input. In addition a
    list of policies may be provided.
    The allocated uid, gids and the poolindex will be returned to the calling
    application.
    This interface is intended to be used by a wrapper function for the 
        DYNAMIC ACCOUNTS SERVICE (DAS).

Parameters:
    user_dn     : the DN of the user
    fqan_list   : the list of (VOMS) FQANs that have been asserted to the user
    nfqan       : the number of FQANs in fqan_list
    request     : RSL string
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation
    puid        : pointer to the uid found (output parameter)
    ppgid_list  : pointer to the list of primary gids found (output parameter)
    psgid_list  : pointer to the list of secondary gids found (output parameter)
    poolindexp  : pointer to poolindex string (output parameter)

Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
/*!
    \fn lcmaps_run_with_fqans_and_return_account(
        char *            user_dn,
        char **           fqan_list,
        int               nfqan,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames,
        uid_t *           puid,
        gid_t **          ppgid_list,
        int *             pnpgid,
        gid_t **          psgid_list,
        int *             pnsgid,
        char **           poolindexp
    )
    \brief let LCMAPS handle the user mapping based on fqans and
           return the account information

    LCMAPS will run with a list of FQANs and the DN as an input. In addition a
    list of policies may be provided.
    The allocated uid, gids and the poolindex will be returned to the calling
    application.
    This interface is intended to be used by a wrapper function for the 
        \b WorkSpace \b Service (WSS, f.q.a. Dynamic Account Service - DAS).

    \param user_dn     the DN of the user
    \param fqan_list   the list of (VOMS) FQANs that have been asserted to the user
    \param nfqan       the number of FQANs in fqan_list
    \param request     RSL string
    \param npols       number of policies to be considered for evaluation
    \param policynames the names of the policies to be considered for evaluation
    \param puid        pointer to the uid found (output parameter)
    \param ppgid_list  pointer to the list of primary gids found (output parameter)
    \param pnpgid      pointer to the number of primary gids found (output parameter)
    \param psgid_list  pointer to the list of secondary gids found (output parameter)
    \param pnsgid      pointer to the number of secondary gids found (output parameter)
    \param poolindexp  pointer to poolindex string (output parameter)

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_with_fqans_and_return_account(
        char *            user_dn,
        char **           fqan_list,
        int               nfqan,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames,
        uid_t *           puid,
        gid_t **          ppgid_list,
        int *             pnpgid,
        gid_t **          psgid_list,
        int *             pnsgid,
        char **           poolindexp
);


#ifdef LCMAPS_GSI_MODE
/******************************************************************************
Function:   lcmaps_run
Description:
    Do the user mapping based on the user's gss (gsi) credential and the job
    request.
    This is the legacy lcmaps interface and is used by
        the GATEKEEPER.

Parameters:
    request: RSL string
    user_cred : user globus credential handle
Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
#if ALLOW_EMPTY_CREDENTIALS
/*!
    \fn lcmaps_run(
        char * user_dn_tmp,
        gss_cred_id_t user_cred,
        lcmaps_request_t request
    )
    \brief let LCMAPS handle the user mapping

    Do the user mapping based on the user's gss (gsi) credential and the job
    request.
    As a back-up for empty credentials the user DN may be specified as well.
    For this \b ALLOW_EMPTY_CREDENTIALS should be defined.
    This is the legacy lcmaps interface and is used by
        the \b gatekeeper.

    \param user_dn_tmp user DN
    \param user_cred   GSS/GSI user credential
    \param request     authorization request as RSL string

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run(
        char * user_dn_tmp,
        gss_cred_id_t user_cred,
        lcmaps_request_t request
);
#else
/*!
    \fn lcmaps_run(
        gss_cred_id_t user_cred,
        lcmaps_request_t request
    )
    \brief let LCMAPS handle the user mapping

    Do the user mapping based on the user's gss (gsi) credential and the job
    request.
    This is the legacy lcmaps interface and is used by
        the \b gatekeeper.

    \param user_cred GSS/GSI user credential
    \param request   authorization request as RSL string

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run(
        gss_cred_id_t user_cred,
        lcmaps_request_t request
);
#endif


/******************************************************************************
Function:   lcmaps_run_and_return_username
Description:
    do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return user name.
    This interface is used by the
        GRIDFTP SERVER.

Parameters:
    request     : RSL string
    user_cred   : user globus credential handle
    usernamep   : pointer to user name (to be freed by calling application)
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation

Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
#if ALLOW_EMPTY_CREDENTIALS
/*!
    \fn lcmaps_run_and_return_username(
        char * user_dn_tmp,
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** usernamep,
        int npols,
        char ** policynames
    )
    \brief let LCMAPS handle the user mapping and return user name

    do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return user name.
    As a back-up for empty credentials the user DN may be specified as well.
    For this \b ALLOW_EMPTY_CREDENTIALS should be defined.
    This interface is used by the
        \b GridFTP server.

    \param user_dn_tmp user DN
    \param user_cred   GSS/GSI user credential
    \param request     authorization request as RSL string
    \param usernamep   pointer to user name (to be freed by calling application).
                       Note: usernamep should be non-NULL at the start !
    \param npols       number of policies to be considered for evaluation
    \param policynames the names of the policies to be considered for evaluation

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/

LCMAPS_C_STORAGE_CLASS int lcmaps_run_and_return_username(
        char * user_dn_tmp,
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** usernamep,
        int npols,
        char ** policynames
);
#else
/*!
    \fn lcmaps_run_and_return_username(
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** usernamep,
        int npols,
        char ** policynames
    )
    \brief let LCMAPS handle the user mapping and return user name

    do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return user name.
    This interface is used by the
        \b GridFTP server.

    \param user_cred   GSS/GSI user credential
    \param request     authorization request as RSL string
    \param usernamep   pointer to user name (to be freed by calling application).
                       Note: usernamep should be non-NULL at the start !
    \param npols       number of policies to be considered for evaluation
    \param policynames the names of the policies to be considered for evaluation

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/

LCMAPS_C_STORAGE_CLASS int lcmaps_run_and_return_username(
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** usernamep,
        int npols,
        char ** policynames
);
#endif


/******************************************************************************
Function:   lcmaps_run_and_return_poolindex
Description:
    do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return the poolindex
    This interface was intended to be used by a wrapper function for the 
        DYNAMIC ACCOUNTS SERVICE (DAS).

Parameters:
    request     : RSL string
    user_cred   : user globus credential handle
    poolindexp  : pointer to poolindex (to be freed by calling application).
                  Note: poolindexp should be non-NULL at the start !
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation

Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
#if ALLOW_EMPTY_CREDENTIALS
/*!
    \fn lcmaps_run_and_return_poolindex(
        char * user_dn_tmp,
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** poolindexp,
        int npols,
        char ** policynames
    )
    \brief let LCMAPS handle the user mapping and return a poolindex

    do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return the poolindex
    As a back-up for empty credentials the user DN may be specified as well.
    For this \b ALLOW_EMPTY_CREDENTIALS should be defined.
    This interface was intended to be used by a wrapper function for the 
        \b WorkSpace \b Service (WSS, f.q.a. Dynamic Account Service - DAS).

    \param user_dn_tmp user DN
    \param user_cred   GSS/GSI user credential
    \param request     authorization request as RSL string
    \param poolindexp  pointer to poolindex (to be freed by calling application).
                       Note: poolindex should be non-NULL at the start !
    \param npols       number of policies to be considered for evaluation
    \param policynames the names of the policies to be considered for evaluation

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_and_return_poolindex(
        char * user_dn_tmp,
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** poolindexp,
        int npols,
        char ** policynames
);
#else
/*!
    \fn lcmaps_run_and_return_poolindex(
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** poolindexp,
        int npols,
        char ** policynames
    )
    \brief let LCMAPS handle the user mapping and return a poolindex

    Do the user mapping based on the provided list of policies (first successful
    policy found in the lcmaps policy file (lcmaps.db) will result in the user
    mapping) and return the poolindex
    This interface was intended to be used by a wrapper function for the 
        \b WorkSpace \b Service (WSS, f.q.a. Dynamic Account Service - DAS).

    \param user_cred   GSS/GSI user credential
    \param request     authorization request as RSL string
    \param poolindexp  pointer to poolindex (to be freed by calling application).
                       Note: poolindex should be non-NULL at the start !
    \param npols       number of policies to be considered for evaluation
    \param policynames the names of the policies to be considered for evaluation

    \retval 0 mapping succeeded.
    \retval 1 mapping failed.
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_and_return_poolindex(
        gss_cred_id_t user_cred,
        lcmaps_request_t request,
        char ** poolindexp,
        int npols,
        char ** policynames
);
#endif

/******************************************************************************
Function:   lcmaps_run_with_pem_and_return_account
Description:
    LCMAPS runs receiving a proxy credential in a PEM formatted string.
    A list of policies may be provided.
    The allocated uid, gids and the poolindex will be returned to the calling
    application.
    This interface is intended to be used by the modified suexec wrapper.

Parameters:
    user_dn     : the DN of the user
    pem_string: : the PEM-encoded string containing the user proxy
    mapcounter: : The counter which will be added to the poolindex, effectively enabling
                  multiple account mappings
    request     : RSL string
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation
    puid        : pointer to the uid found (output parameter)
    ppgid_list  : pointer to the list of primary gids found (output parameter)
    pnpgid      : pointer to the number of primary gids found (output parameter)
    psgid_list  : pointer to the list of secondary gids found (output parameter)
    pnsgid      : pointer to the number of secondary gids found (output parameter)
    poolindexp  : pointer to poolindex string (output parameter)

Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
/*!
    \fn lcmaps_run_with_pem_and_return_account(
        char *            user_dn,
        char *            pem_string,
        int               mapcounter,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames,
        uid_t *           puid,
        gid_t **          ppgid_list,
        int *             pnpgid,
        gid_t **          psgid_list,
        int *             pnsgid,
        char **           poolindexp
    )
    \brief Do the mapping based on the user's credential in PEM-string format

    LCMAPS runs receiving a proxy credential in a PEM formatted string.
    A list of policies may be provided.
    The allocated uid, gids and the poolindex will be returned to the calling
    application.
    This interface is intended to be used by the modified suexec wrapper.

    \param user_dn     The DN of the user
    \param pem_string  The PEM-encoded string containing the user proxy
    \param mapcounter: The counter which will be added to the poolindex, effectively enabling
                       multiple account mappings
    \param request     RSL string
    \param npols       Number of policies to be considered for evaluation
    \param policynames The names of the policies to be considered for evaluation
    \param puid        Pointer to the uid found (output parameter)
    \param ppgid_list  Pointer to the list of primary gids found (output parameter)
    \param pnpgid      Pointer to the number of primary gids found (output parameter)
    \param psgid_list  Pointer to the list of secondary gids found (output parameter)
    \param pnsgid      Pointer to the number of secondary gids found (output parameter)
    \param poolindexp  Pointer to poolindex string (output parameter)

    \retval 0 mapping succeeded
    \retval 1 mapping failed
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_with_pem_and_return_account(
        char *            user_dn,
        char *            pem_string,
        int               mapcounter,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames,
        uid_t *           puid,
        gid_t **          ppgid_list,
        int *             pnpgid,
        gid_t **          psgid_list,
        int *             pnsgid,
        char **           poolindexp
);

/******************************************************************************
Function:   lcmaps_run_and_verify_account_from_pem
Description:
    LCMAPS verifies the account mapping based on the proxy credential
    in a PEM formatted string.
    A list of policies may be provided.
    This interface is intended to be used by the modified suexec wrapper.

Parameters (only input):
    user_dn     : the user DN
    pem_string: : the PEM-encoded string containing the user proxy
    uid         : the uid of the account that should be verified
    pgid_list   : the list of primary gids of the account that should be verified
    npgid       : the number of primary gids of the account that should be verified
    sgid_list   : the list of secondary gids of the account that should be verified
    nsgid       : the number of secondary gids of the account that should be verified
    poolindex   : poolindex string of the account that should be verified
    request     : RSL string
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation

Returns:
    0: verification succeeded
    1: verification failed
******************************************************************************/
/*!
    \fn lcmaps_run_and_verify_account_from_pem(
        char *            user_dn,
        char *            pem_string,
        uid_t             uid,
        gid_t *           pgid_list,
        int               npgid,
        gid_t *           sgid_list,
        int               nsgid,
        char *            poolindex,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames
    )
    \brief Verify the account mapping

    LCMAPS verifies the account mapping based on the proxy credential
    in a PEM formatted string.
    A list of policies may be provided.
    This interface is intended to be used by the modified suexec wrapper.

    \param user_dn      the user DN (input)
    \param pem_string:  the PEM-encoded string containing the user proxy (input)
    \param uid          the uid of the account that should be verified (input)
    \param pgid_list    the list of primary gids of the account that should be verified (input)
    \param npgid        the number of primary gids of the account that should be verified (input)
    \param sgid_list    the list of secondary gids of the account that should be verified (input)
    \param nsgid        the number of secondary gids of the account that should be verified (input)
    \param poolindex    poolindex string of the account that should be verified (input)
    \param request      RSL string (input)
    \param npols        number of policies to be considered for evaluation (input)
    \param policynames  the names of the policies to be considered for evaluation (input)

    \retval 0 verification succeeded
    \retval 1 verification failed
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_and_verify_account_from_pem(
        char *            user_dn,
        char *            pem_string,
        uid_t             uid,
        gid_t *           pgid_list,
        int               npgid,
        gid_t *           sgid_list,
        int               nsgid,
        char *            poolindex,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames
);

/******************************************************************************
Function:   lcmaps_run_with_stack_of_x509_and_return_account
Description:
    LCMAPS runs receiving a certificate chain, containing at least an End-Entity Certificate.
    A list of policies may be provided.
    The allocated uid, gids and the poolindex will be returned to the calling
    application.

Parameters:
    cert_chain  : The certificate chain to use for the mapping
    mapcounter: : The counter which will be added to the poolindex, effectively enabling
                  multiple account mappings
    request     : RSL string
    npols       : number of policies to be considered for evaluation
    policynames : the names of the policies to be considered for evaluation
    puid        : pointer to the uid found (output parameter)
    ppgid_list  : pointer to the list of primary gids found (output parameter)
    pnpgid      : pointer to the number of primary gids found (output parameter)
    psgid_list  : pointer to the list of secondary gids found (output parameter)
    pnsgid      : pointer to the number of secondary gids found (output parameter)
    poolindexp  : pointer to poolindex string (output parameter)

Returns:
    0: mapping succeeded
    1: mapping failed
******************************************************************************/
/*!
    \fn lcmaps_run_with_stack_of_x509_and_return_account(
        STACK_OF (X509) * cert_chain,
        uid_t             uid,
        gid_t *           pgid_list,
        int               npgid,
        gid_t *           sgid_list,
        int               nsgid,
        char *            poolindex,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames
    )
    \brief Run LCMAPS and map to an account based on the supplied certificate chain.

    LCMAPS runs receiving a certificate chain, containing at least an End-Entity Certificate.
    A list of policies may be provided. The allocated uid, gids and the poolindex will be returned to the calling
    application.

    \param cert_chain   The certificate chain to use for the mapping
    \param uid          the uid of the account that should be verified (input)
    \param pgid_list    the list of primary gids of the account that should be verified (input)
    \param npgid        the number of primary gids of the account that should be verified (input)
    \param sgid_list    the list of secondary gids of the account that should be verified (input)
    \param nsgid        the number of secondary gids of the account that should be verified (input)
    \param poolindex    poolindex string of the account that should be verified (input)
    \param request      RSL string (input)
    \param npols        number of policies to be considered for evaluation (input)
    \param policynames  the names of the policies to be considered for evaluation (input)

    \retval 0 mapping succeeded
    \retval 1 mapping failed
*/
LCMAPS_C_STORAGE_CLASS int lcmaps_run_with_stack_of_x509_and_return_account(
        STACK_OF(X509) *  cert_chain,
        int               mapcounter,
        lcmaps_request_t  request,
        int               npols,
        char **           policynames,
        uid_t *           puid,
        gid_t **          ppgid_list,
        int *             pnpgid,
        gid_t **          psgid_list,
        int *             pnsgid,
        char **           poolindexp
);
#endif /* LCMAPS_GSI_MODE*/


LCMAPS_C_STORAGE_CLASS int lcmaps_get_major_version (void);
LCMAPS_C_STORAGE_CLASS int lcmaps_get_minor_version (void);
LCMAPS_C_STORAGE_CLASS int lcmaps_get_patch_version (void);


#endif /* LCMAPS_H */

/******************************************************************************
CVS Information:
    $Source: /srv/home/dennisvd/svn/mw-security/lcmaps/interface/lcmaps.h,v $
    $Date: 2011-03-04 21:55:13 +0100 (Fri, 04 Mar 2011) $
    $Revision: 14868 $
    $Author: dennisvd $
******************************************************************************/
