/**
 * Copyright (c) Members of the EGEE Collaboration. 2004-2010. 
 * See http://www.eu-egee.org/partners/ for details on the copyright
 * holders.  
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); 
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 *
 *
 *  Authors:
 *  2009-
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     Mischa Sall\'e <msalle@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *     <grid-mw-security@nikhef.nl> 
 *
 *  2007-2009
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 *  2003-2007
 *     Martijn Steenbakkers <martijn@nikhef.nl>
 *     Gerben Venekamp <venekamp@nikhef.nl>
 *     Oscar Koeroo <okoeroo@nikhef.nl>
 *     David Groep <davidg@nikhef.nl>
 *     NIKHEF Amsterdam, the Netherlands
 *
 */


/*!
    \file   lcmaps_return_poolindex.h
    \brief  This interface of LCMAPS takes the user gsi credential as input and
            returns the uid, gids and poolindex, i.e. leaseid.
    \author Martijn Steenbakkers for EGEE.

    This header contains the following LCMAPS library functions:
    -# lcmaps_return_poolindex
        Returns the poolindex based on fixed arguments:
                security context, buffer and buffer length
    -# lcmaps_return_poolindex_from_gss_cred
        Returns the poolindex based on fixed arguments:
                gss credential, buffer and buffer length

    In general the behaviour of LCMAPS can be influenced by setting the following
    environment variables:
        Env variable                        Description
        (default value)
        ---------------                     -----------
        LCMAPS_LOG_FILE                     The logfile to which lcmaps log
        ("/var/log/lcmaps.log")             messages will be written

        LCMAPS_DB_FILE                      the location (name) of the lcmaps policy file
        (LCMAPS_ETC_DIR"/lcmaps.db")

        LCMAPS_POLICY_NAME                  the name of the lcmaps policy (in the
        ("acquisition_policy")              policy file) that should be evaluated

        LCMAPS_LOG_STRING                   A string that will be prepended to all
        ("poolindex :")                     LCMAPS logging messages.

        LCMAPS_DEBUG_LEVEL                  The debug level influences the logging
        ("0")                               verbosity. values 0-5.

        LCMAPS_MODULES_DIR                  The directory in which modules can be found.
        ("")                                Note: may be overwritten by what is in
                                            the policy file (the convential method)

        LCMAPS_ETC_DIR                      The directory of the lcmaps policy files
        ("/opt/glite/etc/lcmaps")           and other config files.

        GRIDMAPDIR                          The directory in which the poolaccount leases
        ("")                                are registered.
                                            Note: may be overwritten by what is in
                                            the policy file (the convential method)

    \ingroup LcmapsInterface
*/

#ifndef LCMAPS_RETURN_POOLINDEX_H
#define LCMAPS_RETURN_POOLINDEX_H

#ifndef LCMAPS_RETURN_POOLINDEX_C
#   define LCMAPS_RETURN_POOLINDEX_C_STORAGE_CLASS extern
#else
#   define LCMAPS_RETURN_POOLINDEX_C_STORAGE_CLASS
#endif

/******************************************************************************
                             Include header files
******************************************************************************/
#include <gssapi.h>

/******************************************************************************
 *                 Module definition
 *****************************************************************************/

/******************************************************************************
Function:   lcmaps_return_poolindex
Description:
    If LCMAPS is invoked via this interface it will return the poolindex,
    alternatively named leaseid, to the calling application.

Parameters:
    context_handle:
        The GSS Security context established during invocation of the calling
        service. This parameter is of type gss_ctx_id_t.
    poolindex_buffer:
        A pointer to a buffer. This buffer will contain the poolindex (NUL
        terminated string) upon successful return.
        This parameter is of type char *.
    buffer_length:
        The length of the above mentioned buffer.
        This parameter is of type unsigned int.

Returns:
    0: success
    1: failure
******************************************************************************/
/*!
    \fn lcmaps_return_poolindex(
        gss_ctx_id_t context_handle,
        char *       poolindex_buffer,
        unsigned int buffer_length
    )
    \brief LCMAPS returns the poolindex
    If LCMAPS is invoked via this interface it will return the poolindex,
    alternatively named leaseid, to the calling application.

    \param context_handle
           The GSS Security context established during invocation of the calling
           service.
    \param poolindex_buffer
           A pointer to a buffer. This buffer will contain the poolindex (NUL
           terminated string) upon successful return.
    \param buffer_length
           The length of the above mentioned buffer.

    \retval 0 success.
    \retval 1 failure.
*/
LCMAPS_RETURN_POOLINDEX_C_STORAGE_CLASS int
lcmaps_return_poolindex(
    gss_ctx_id_t context_handle,
    char *       poolindex_buffer,
    unsigned int buffer_length
);

/******************************************************************************
Function:   lcmaps_return_poolindex_from_gss_cred
Description:
    If LCMAPS is invoked via this interface it will return the poolindex,
    alternatively named leaseid, to the calling application.
    Instead of using the security context as an input it uses a gss credential
    (type gss_cred_id_t).

Parameters:
    credential_handle:
        The GSS credential (of type gss_cred_id_t) of the user.
    poolindex_buffer:
        A pointer to a buffer. This buffer will contain the poolindex (NUL
        terminated string) upon successful return.
        This parameter is of type char *.
    buffer_length:
        The length of the above mentioned buffer.
        This parameter is of type unsigned int.

Returns:
    0: success
    1: failure
******************************************************************************/
/*!
    \fn lcmaps_return_poolindex_from_gss_cred(
        gss_cred_id_t  credential_handle,
        char *         poolindex_buffer,
        unsigned int   buffer_length
    )
    \brief LCMAPS returns the poolindex
    If LCMAPS is invoked via this interface it will return the poolindex,
    alternatively named leaseid, to the calling application.
    Instead of using the security context as an input it uses a gss credential
    (type gss_cred_id_t).

    \param credential_handle
           The GSS credential (of type gss_cred_id_t) of the user.
    \param poolindex_buffer
           A pointer to a buffer. This buffer will contain the poolindex (NUL
           terminated string) upon successful return.
    \param buffer_length
           The length of the above mentioned buffer.

    \retval 0 success.
    \retval 1 failure.
*/
LCMAPS_RETURN_POOLINDEX_C_STORAGE_CLASS int
lcmaps_return_poolindex_from_gss_cred(
    gss_cred_id_t credential_handle,
    char *        poolindex_buffer,
    unsigned int  buffer_length
);

#endif /* LCMAPS_RETURN_POOLINDEX_H */
